//
// This code is part of https://demo.grapecity.com.cn/gcword.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using System.Linq;
using System.Globalization;
using GrapeCity.Documents.Word;

namespace DsWordWeb.Demos
{
    // This example shows how to deal with the template formatter
    // conflicting with another formatter error.
    public class DataTplFixFormatterConflict
    {
        // Code demonstrating the problem:
        GcWordDocument Problem()
        {
            using var oceans = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            var doc = new GcWordDocument();
            doc.DataTemplate.DataSources.Add("ds", oceans);
            // Incorrect: pbb (paragraph-block-behavior) and rbb (run-block-behavior) formatters
            // were both applied to the same range template tag:
            doc.Body.Paragraphs.Add("{{#ds.seas}:pbb():rbb()}{{ds.seas.name}} {{/ds.seas}}");
            doc.DataTemplate.Process(CultureInfo.GetCultureInfo("en-US"));
            return doc;
        }

        // Code demonstrating the fix:
        GcWordDocument Fix()
        {
            using var oceans = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            var doc = new GcWordDocument();
            doc.DataTemplate.DataSources.Add("ds", oceans);
            // Correct: pbb and rbb are block-modifier formatters, so only one of them can be used on a single range template tag:
            doc.Body.Paragraphs.Add("{{#ds.seas}:rbb()}{{ds.seas.name}} {{/ds.seas}}");
            doc.DataTemplate.Process(CultureInfo.GetCultureInfo("en-US"));
            return doc;
        }

        public GcWordDocument CreateDocx()
        {
            GcWordDocument doc;
            try
            {
                // This fails:
                doc = Problem();
            }
            catch (Exception ex)
            {
                // This works:
                doc = Fix();
                // Insert a brief explanation of the problem and the fix into the generated document:
                doc.Body.Paragraphs.Insert(
                    $"The error \"{ex.Message}\" occurred because a pbb (paragraph-block-behavior) and a rbb (run-block-behavior) " +
                    $"formatters were both applied to the same range template tag. pbb and rbb are block-modifier formatters, " +
                    $"so only one of them can be used on a single range template tag.",
                    doc.Styles[BuiltInStyleId.BlockText],
                    InsertLocation.Start);
            }
            return doc;
        }
    }
}
